
control_plane_labels=(
    "node-role.kubernetes.io/master"
    "node-role.kubernetes.io/control-plane"
    "node.kubernetes.io/role=master"
    "node.kubernetes.io/microk8s-controlplane"
)

for label in "${control_plane_labels[@]}"; do
    master_node=$(kubectl get nodes -o custom-columns=NAME:.metadata.name --no-headers -l "$label")
    if [ -n "$master_node" ]; then
        worker_nodes=($(kubectl get nodes -o custom-columns=NAME:.metadata.name --no-headers -l "!$label"))
        break
    fi
done

selector1="${worker_nodes[0]}-lpm"
selector2="${worker_nodes[1]}-lpm"

if [[ "${#worker_nodes[@]}" -lt 2 ]]; then
  selector2="${master_node}-lpm"
fi

NAMESPACE='he-codeco-netma'

LPM_C1_POD="$(kubectl get pods -n $NAMESPACE --selector=app=$selector1 -o jsonpath='{.items[0].metadata.name}')"
LPM_C2_POD="$(kubectl get pods -n $NAMESPACE --selector=app=$selector2 -o jsonpath='{.items[0].metadata.name}')"

echo "Using pods $LPM_C1_POD and $LPM_C2_POD for testing."

kubectl exec -n $NAMESPACE $LPM_C1_POD -- bash -c "apt update && apt install -y iputils-ping iproute2 tcpdump" > /dev/null 2>&1
kubectl exec -n $NAMESPACE $LPM_C2_POD -- bash -c "apt update && apt install -y iputils-ping iproute2 tcpdump" > /dev/null 2>&1

NETWORK_INFO_C1=$(kubectl exec -n $NAMESPACE $LPM_C1_POD -- ip -o -4 addr show | awk '{print $2, $4}')
NETWORK_INFO_C2=$(kubectl exec -n $NAMESPACE $LPM_C2_POD -- ip -o -4 addr show | awk '{print $2, $4}')

while IFS= read -r line; do
    INTERFACE=$(echo "$line" | awk '{print $1}')
    IP_ADDRESS=$(echo "$line" | awk '{print $2}')
    if [[ "$IP_ADDRESS" == 10.0.0.* ]]; then
         LPM_C1_INTERFACE=$INTERFACE
         LPM_C1_IP=$(echo $IP_ADDRESS | sed 's#/.*##')
         echo "Interface: $INTERFACE, IP Address: $LPM_C1_IP"
         break
    fi
done <<< "$NETWORK_INFO_C1"

while IFS= read -r line; do
    INTERFACE=$(echo "$line" | awk '{print $1}')
    IP_ADDRESS=$(echo "$line" | awk '{print $2}')
    if [[ "$IP_ADDRESS" == 10.0.0.* ]]; then
         LPM_C2_INTERFACE=$INTERFACE
         LPM_C2_IP=$(echo $IP_ADDRESS | sed 's#/.*##')
         echo "Interface: $INTERFACE, IP Address: $LPM_C2_IP"
         break
    fi
done <<< "$NETWORK_INFO_C2"


echo "performing ping and tcpdump through l2sm interfaces..."
kubectl exec -n $NAMESPACE $LPM_C1_POD -- bash -c "ping -c 10 $LPM_C2_IP " > ./ping_c1.txt 2>&1 &
kubectl exec -n $NAMESPACE $LPM_C2_POD -- bash -c "timeout 15 tcpdump -i $LPM_C2_INTERFACE icmp" > ./tcpdump_c2.txt  2>&1


pings=$(grep -c "icmp_seq" ping_c1.txt)
tcpdump_requests=$(grep "ICMP echo request" tcpdump_c2.txt  | grep $LPM_C1_IP | grep -c $LPM_C2_IP)
tcpdump_replies=$(grep "ICMP echo reply" tcpdump_c2.txt | grep $LPM_C1_IP | grep -c $LPM_C2_IP)

echo "Performed $pings pings from 1st lpm pod"
echo "Received $tcpdump_requests ICMP requests and $tcpdump_replies ICMP replies in 2nd lpm pod"

if [[ "$pings" > 0 && "$tcpdump_replies" > 0 ]]; then
    echo "Communication between l2sm interfaces is confirmed"
else
    echo "Communication between l2sm interfaces failed."
fi

rm ./ping_c1.txt ./tcpdump_c2.txt
