
node_names_netma=($(kubectl get netma-topology netma-sample -o yaml -n he-codeco-netma | yq '.underlay-topology.nodes[].name'))
node_BWs_netma=($(kubectl get netma-topology netma-sample -o yaml -n he-codeco-netma | yq '.underlay-topology.nodes[].capabilities.uNodeBandWidth'))

node_names=($(kubectl get nodes -o custom-columns=NAME:.metadata.name --no-headers))

### CHECK IF UNDERLAY TOPOLOGY CONTAINS THE CORRECT NODE NAMES

sorted_node_names=($(printf "%s\n" "${node_names[@]}" | sort))
sorted_node_names_netma=($(printf "%s\n" "${node_names_netma[@]}" | sort))

if [[ "${sorted_node_names[*]}" == "${sorted_node_names_netma[*]}" ]]; then
    echo "NetMA successfully recognises nodes ${node_names_netma[@]}"
else
    echo "NetMA failed to indentify the cluster's underlay topology :("
    exit 0
fi

### CHECK THE NODES' BANDWIDTH

if [[ "${#node_BWs_netma[@]}" -lt "${#node_names[@]}" ]]; then
    echo "Did not find ndoes' Bandwidths"
    exit 1
fi

# Regular expression for a float number
float_regex='^[+-]?[0-9]*\.?[0-9]+$'

# Flag to track if all elements are valid floats
all_floats=true

# Check each bandwidth in the list
for bw in "${node_BWs_netma[@]}"; do
    if [[ ! $bw =~ $float_regex ]]; then
        echo "Invalid float: $bw"
        all_floats=false
    else
        if awk "BEGIN {exit !($bw == 0)}"; then
            echo "Warning: A node has zero bandwidth"
        fi

    fi
done

if [[ all_floats -eq true ]]; then
    echo "Nodes' bandwidths seem correct"
else
    echo "Nodes have invalid bandwidths: ${node_BWs_netma[@]}"
fi


### check overlay topology

links_names=($(kubectl get netma-topology netma-sample -o yaml -n he-codeco-netma | yq '.overlay-topology.links[].name'))
links_sources=($(kubectl get netma-topology netma-sample -o yaml -n he-codeco-netma | yq '.overlay-topology.links[].source'))
links_targets=($(kubectl get netma-topology netma-sample -o yaml -n he-codeco-netma | yq '.overlay-topology.links[].target'))
links_latency=($(kubectl get netma-topology netma-sample -o yaml -n he-codeco-netma | yq '.overlay-topology.links[].capabilities.oLatencyNanos'))

if [[ "${#links_sources[@]}" -lt "${#node_names[@]}" ]]; then
    echo "overlay topology links not found :("
    exit 1
fi

echo "Found the following links:"
for i in "${!links_sources[@]}"; do
    echo "    Name:  ${links_names[$i]}, Source: ${links_sources[$i]}, Target: ${links_targets[$i]}, Latency: ${links_latency[$i]} "
done
